// ##########################################################################
// ####                                                                  ####
// ####      WarpEd - A portable text editor for OS/2 Warp and Linux     ####
// ####    ===========================================================   ####
// ####                                                                  ####
// #### command.cc                                                       ####
// ####                                                                  ####
// #### Version 0.80  --  October 06, 2000                               ####
// ####                                                                  ####
// #### Copyright (C) 1996  Thomas Dreibholz                             ####
// ####               1997  Molbachweg 7                                 ####
// ####                     51674 Wiehl/Germany                          ####
// ####                     EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "const.h"

#include "edit.h"
#include "edit.icc"
#include "terminal.h"
#include "str.h"
#include "row.h"
#include "row.icc"
#include "system.h"

#include "command.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>



int gReplaceMode = 0;

extern int         gLogo;
extern ColorPrefs  gWarpEdColors;
extern WarpEd     *gWarpEd;


// #######################################################################
// ===== Info-Zeile und Logo =============================================
// #######################################################################


/* Info-Zeile aktualisieren */
void UpdateInfo(char *info, int status, int flags)
{
   int     i,j;
   time_t  systime;
   tm     *tm;

   for(i = 0;i < gWarpEd->Terminal.zWidth - 1;i++) {
      gWarpEd->Buffer[0][i] = ' ';
   }
   gWarpEd->Buffer[0][i] = 0x00;

   // Wenn kein Status-Update: Alten Info-Text benutzen
   if(!(flags & cUpdateStatus)) { 
      info = gWarpEd->OldInfoText;
      if((info != NULL) && (info[0] == '$')) {
         info = &info[1];
      }
   }
   else {
      // Kein Fehler: Angegebenen Info-Text benutzen
      if(status != 0) {
         info = gWarpEdStatus[status];
         gWarpEd->OldInfoText = NULL;
      } 
      else {
         gWarpEd->OldInfoText = info;   
      }
   }

   if(info == NULL) {
      strcpy(gWarpEd->Buffer[1]," WarpEd 0.80 ");
      gWarpEd->OldInfoText = NULL;
   }
   else {
      if(info[0] == '$') {
         gWarpEd->Terminal.Bell();    
         strcpy(gWarpEd->Buffer[1],&info[1]);
      }
      else {
         strcpy(gWarpEd->Buffer[1],info);
      }
   }
   for(i = 0;i < strlen(gWarpEd->Buffer[1]);i++)
      gWarpEd->Buffer[0][i] = gWarpEd->Buffer[1][i];

   time(&systime);
   tm   = localtime(&systime);

   if(flags & cUpdateTimeAndCursor) {
      #if defined OS2_WARP
         sprintf(gWarpEd->Buffer[1],"%05d:%03d   %2d:%02d:%02d %s",
                 gWarpEd->Ed.CursorY() + 1,gWarpEd->Ed.CursorX() + 1,
                 tm->tm_hour,tm->tm_min,tm->tm_sec,tzname[0]);
         j = strlen(gWarpEd->Buffer[1]);
         for(i = 0;i < j;i++) {
            gWarpEd->Buffer[0][gWarpEd->Terminal.zWidth - 1 - j + i] = gWarpEd->Buffer[1][i];
         }
      #endif
      #if defined LINUX
         sprintf(gWarpEd->Buffer[1],"%05d:%03d   %2d:%02d:%02d %s ",
                 gWarpEd->Ed.CursorY() + 1,gWarpEd->Ed.CursorX() + 1,
                 tm->tm_hour,tm->tm_min,tm->tm_sec,tzname[0]);
         j = strlen(gWarpEd->Buffer[1]);
         for(i = 0;i < j;i++) {
            gWarpEd->Buffer[0][gWarpEd->Terminal.zWidth - j + i] = gWarpEd->Buffer[1][i];
         }
      #endif

   }

   gWarpEd->Terminal.BeginWriting();
   gWarpEd->Terminal.WriteString(0,gWarpEd->Terminal.zHeight - 1,
                        gWarpEdColors.InfoStyle,
                        gWarpEdColors.InfoFrontPen,
                        gWarpEdColors.InfoBackPen,
                        gWarpEd->Buffer[0]);
   gWarpEd->Terminal.EndWriting();
}


// Logo anzeigen
void ShowLogo()
{
   gWarpEd->Terminal.BeginWriting();
   int x = (gWarpEd->Terminal.zWidth - strlen(gWarpEdLogo[0])) / 2;
   int y = (gWarpEd->Terminal.zHeight - 1 - gWarpEdLogoHeight) / 2;
   for(int i = 0;i < gWarpEdLogoHeight;i++) {
      gWarpEd->Terminal.WriteString(x,i + y,
                           gWarpEdColors.Style,
                           gWarpEdColors.FrontPen,gWarpEdColors.BackPen,
                           gWarpEdLogo[i]);
   }
   gWarpEd->Terminal.EndWriting();
   gLogo = 1;
}


// #######################################################################
// ===== Kommandozeilen-Auswertung =======================================
// #######################################################################


// Ed beenden
int Bye(char *arg, int flags)
{ 
   return(TRUE); 
}


// Text speichern
// Flags 0 = Save
//       1 = Save as
//       2 = Save and Exit
int Save(char *arg, int flags)
{
   char *name;

   // Name ermitteln
   if(flags == 1) {
      name = arg;
   }
   else {
      name = gWarpEd->FileName;
   }

   // Name berprfen
   if(name[0] == 0x00) {
      gWarpEd->Ed.status = Editor::ioError;
      return(FALSE);
   }

   // Text speichern 
   if(gWarpEd->Ed.SaveText(name) == true) {
      strcpy(gWarpEd->FileName,name);
   }
  
   // Ed bei Save and Exit beenden
   if((flags == 2) && (gWarpEd->Ed.status == 0)) {
      return(TRUE);
   }
  
   return(FALSE);
}


// Text laden
int Load(char *arg, int flags)
{
   if(arg[0] != 0x00) {
      if(gWarpEd->Ed.LoadText(arg) == true) {
         strcpy(gWarpEd->FileName,arg);
      }
   }
   else gWarpEd->Ed.status = Editor::ioError;
   return(FALSE);
}


// Neuen Text beginnen
int New(char *arg, int flags)
{
   gWarpEd->Ed.NewText();
   ShowLogo();
   gWarpEd->FileName[0] = 0x00;
   return(FALSE);
}

// Block in Datei schreiben
int WriteBlock(char *arg, int flags)
{
   if(arg[0] != 0x00) {
      gWarpEd->Ed.CopyBlock(arg);
   }
   else gWarpEd->Ed.status = Editor::ioError;
   return(FALSE);
}


// Datei einfgen
int InsertFile(char *arg, int flags)
{
   if(arg[0] != 0x00) {
      gWarpEd->Ed.InsertBlock(arg);
   }
   else gWarpEd->Ed.status = Editor::ioError;
   return(FALSE);
}


// Suchen
// Flags 0 = Find
//       1 = Replace
int Find(char *arg, int flags)
{
   strcpy(gWarpEd->FindText,arg);
   if(flags == 1) {
      int enter = CommandLine("Replace with> ",gWarpEd->ReplaceText);
      gReplaceMode = 1;
   }
   else {
      gReplaceMode = 0;
   }
   TextFinder(gWarpEd->Ed.CursorY(),gWarpEd->Ed.CursorX(),cFindForward);
   return(FALSE);
}


struct WarpEdCommand
{
   char *Command;
   int   Flags;
   int  (*Routine)(char *arg, int flags);
};


// Befehlstabelle
#define cCommands 19
struct WarpEdCommand CommandTable[cCommands] = {
{"Q",      0,&Bye},
{"X",      2,&Save},
{"QUIT",   0,&Bye},
{"BYE",    0,&Bye},
{"SAVEAS", 1,&Save},
{"SA",     1,&Save},
{"S",      0,&Save},
{"SAVE",   0,&Save},
{"LOAD",   0,&Load},
{"L",      0,&Load},
{"NEW",    0,&New},
{"F",      0,&Find},
{"FIND",   0,&Find},
{"R",      1,&Find},
{"REPLACE",1,&Find},
{"WRITEBLOCK", 0,&WriteBlock},
{"WB",         0,&WriteBlock},
{"INSERTFILE", 0,&InsertFile},
{"IF",         0,&InsertFile},
};


// Befehl ausfhren
int ExecuteCommand(char *cmd, char *arg)
{
   int i;

   #if defined OS2_WARP
      strupr(cmd);
   #endif
   #if defined LINUX
      i = 0;
      while(cmd[i] != 0x00) {
         cmd[i] = toupper(cmd[i]);
         i++;
      }
   #endif

   Optimize(arg);
   for(i = 0;i < cCommands;i++) {
      if(!(strcmp(CommandTable[i].Command,cmd))) {
         return(CommandTable[i].Routine(arg,CommandTable[i].Flags));
      }
   }
   
   UpdateInfo("$Unbekannter Befehl!",0,
              cUpdateStatus|cUpdateTimeAndCursor);

   return(FALSE);
}


// Eingabe von Befehlen
int CommandLine(char *title, char *buffer)
{
   int key;
   int ok;
   int enter;

   gWarpEd->Terminal.BeginWriting();
   gWarpEd->Terminal.WriteString(0,gWarpEd->Terminal.zHeight - 1,
                        gWarpEdColors.InfoStyle,
                        gWarpEdColors.InfoFrontPen,
                        gWarpEdColors.InfoBackPen,
                        title);
   gWarpEd->Terminal.EndWriting();
   EdLine cmd;
   cmd.init(&gWarpEd->Terminal,strlen(title),gWarpEd->Terminal.zHeight - 1,
                 gWarpEd->Terminal.zWidth - strlen(title) - 1,
                 cMaxCommandLength - 2,0,0);
   if(ok == FALSE) {
      gWarpEd->Terminal.Bell();
      return(FALSE);
   }

   cmd.SetText(buffer);
   cmd.SetCursor(strlen(buffer));
   cmd.ShowCursor();
   cmd.Update();

   ok = enter = FALSE;
   while(ok == FALSE) {
      key = gWarpEd->Terminal.WaitForKey(0,750);
      switch(key) {
         case KeyEscape:
           ok = TRUE;
          break;
         case KeyReturn:
           ok    = TRUE;
           enter = TRUE;
          break;
         case KeyCursorLeft:
           cmd.CursorLeft();
          break;
         case KeyCursorRight:
           cmd.CursorRight();
          break;
         case KeyHome:
           cmd.CursorHome();
          break;
         case KeyEnd:
           cmd.CursorEnd();
          break;
         case KeyDelete:
           cmd.Del();
          break;
         case KeyBackspace:
           cmd.Backspace();
          break;
         case KeyTab:
           cmd.InsertChar(' ');
          break;
         case 0:
          break;
         default:
           if((key >= 0) && (key <= 0xff)) {
              cmd.InsertChar((char)key);
           }
      }
   }

   cmd.GetText(buffer);
   UpdateInfo(NULL,0,cUpdateTimeAndCursor|cUpdateStatus);
   gWarpEd->Terminal.ShowSysCursor(
      gWarpEd->Ed.CursorX()-gWarpEd->Ed.OffsetX(),
      gWarpEd->Ed.CursorY()-gWarpEd->Ed.OffsetY());

   return(enter);
}  


/* Kommandozeile zeigen und Befehl einlesen und ausfhren */
int GetCommand(char *title, char *cmdText)
{
   char text[cMaxCommandLength];
   int  enter;
   int  i,j;

   if(cmdText != NULL)
      strcpy(text,cmdText);
   else
      text[0] = 0x00;
   enter = CommandLine(title,text);

   if(enter == TRUE) {
      j = strlen(text); 
      for(i = 0;i < strlen(text);i++) {
         if(text[i] == ' ') {
            text[i] = 0x00;
            break;
         }
      }
      if(i == j) {
         text[i + 1] = 0x00;
      }
      return(ExecuteCommand(text,&text[i+1])); 
   }
   return(FALSE);
}


// #######################################################################
// ===== Control-Sequenzen ===============================================
// #######################################################################


// Control-Sequenz ausfhren
int ControlSequence(int k1, int k2, int *quit)
{
   int next = 0;

   switch(k1) {
      case ControlSeq('C'):
        *quit = GetCommand("Ctrl-C> ","Bye"); 
       break;
      case ControlSeq('F'):
         TextFinder(gWarpEd->Ed.CursorY(),gWarpEd->Ed.CursorX() + 1,cFindForward); 
        break;
      case ControlSeq('D'):
         TextFinder(gWarpEd->Ed.CursorY(),gWarpEd->Ed.CursorX() - 1,cFindBackward); 
        break;
      case ControlSeq('S'):
          GetCommand("Ctrl-S> ","Save ");
        break;
      case ControlSeq('L'):
          GetCommand("Ctrl-L> ","Load ");
        break;
      case ControlSeq('E'):
          GetCommand("Ctrl-E> ","Find ");
        break;

/* // --- Tests --------------------------
      case ControlSeq('T'):
        gWarpEd->Terminal.Clear();
        gWarpEd->Ed.Resize(1,1,78,18);
       break;
      case ControlSeq('Z'):
        gWarpEd->Terminal.Clear();
        gWarpEd->Ed.Resize(3,3,65,12);
       break;
   // ------------------------------------
*/
      case ControlSeq('K'):
         switch(k2) {
            case 0:
              next = k1;
             break;
            case ControlSeq('B'):
              gWarpEd->Ed.BlockStart();
             break;
            case ControlSeq('K'):
              gWarpEd->Ed.BlockEnd();
             break;
            case ControlSeq('P'):
              gWarpEd->Ed.KillBlock();
             break;
            case ControlSeq('Y'):
              gWarpEd->Ed.CutBlock(gWarpEd->TmpName);
             break;
            case ControlSeq('C'):
              gWarpEd->Ed.CopyBlock(gWarpEd->TmpName);
             break;
            case ControlSeq('V'):
              gWarpEd->Ed.InsertBlock(gWarpEd->TmpName);
             break;
         }
       break;
   }
   return(next);
}


// #######################################################################
// ===== Text suchen und ersetzen ========================================
// #######################################################################


/* Textsuche in einer bestimmten Zeile */
int FindInLine(int num, int startx, int direction)   
{
   char  *found;
   int    index;

   gWarpEd->Ed.GetLine(num,gWarpEd->Buffer[0]);
   found = strstr(&gWarpEd->Buffer[0][startx],gWarpEd->FindText);
   if(found != 0) {
      index = (int)found - (int)gWarpEd->Buffer[0];
      gWarpEd->Ed.SetCursor(num,index);
      return(TRUE);
   }
   return(FALSE);
}


/* Textsuche */
int TextFinder(int starty, int startx, int direction)
{
   int i;

   if(!(strcmp(gWarpEd->FindText,""))) {
      return(FALSE);
   }

   if(direction == cFindForward) {
      for(i = starty;i < gWarpEd->Ed.Count();i++) {
         if(FindInLine(i,startx,cFindForward) == TRUE) {
            return(TRUE);  
         }
         startx = 0;
      }
   }
   else {
      for(i = starty;i >= 0;i--) {
         if(FindInLine(i,startx,cFindBackward) == TRUE) { 
            return(TRUE);  
         }
         startx = 0;
      }
   }
 
   UpdateInfo("Keine (weiteren) Textstellen gefunden!",0,
              cUpdateStatus|cUpdateTimeAndCursor);
 
   return(FALSE);
}

