/* row.cc: Verwaltung der Editierzeile */

#include <stdlib.h>

#include "str.h"
#include "terminal.h"
#include "row.h"
#include "row.icc"


// Standard-Konstruktor
EdLine::EdLine()
{
   isGood = false;
}


/* Initialisieren einer EdLine */
void EdLine::init(TerminalClass* term,
               int x, int y, int w,
               int bufsize, int o, int c)
{
   // Grundparameter initialisieren
   leftEdge   = x; 
   topEdge    = y;
   width      = w;
   offset     = o;
   cursor     = c;
   showCursor = false;
   newOffset  = false;
   blockStart = -1;
   blockEnd   = -1;
   fastScroll = 3;
   sysCursorUpdate = true;
   terminal        = term;
   isGood          = false;

   // Speicher fr Text belegen und initialsieren
   text = new char[bufsize];
   if(text == NULL) return;
   text[0] = 0x00;

   maxPos  = bufsize - 1;
   isGood  = true;
}


/* Freigeben einer EdLine */
EdLine::~EdLine() 
{ 
   if(text != NULL) {
      delete text;
      text = NULL;
   }
}


/* Gre und Position ndern */
void EdLine::Resize(int x, int y, int w)
{
   leftEdge = x;
   topEdge  = y;
   width    = w;
}
 

// #######################################################################
// ===== Cursorbewegungen ================================================
// #######################################################################


/* Cursor nach links */
void EdLine::CursorLeft()
{
   if(cursor > 0) {
      cursor--;
      if(cursor < offset) {
         if(fastScroll == 0) {
            offset--;
         }
         else {
            offset -= (width / fastScroll);
            if(offset < 0) {
               offset = 0;
            }
         }

         newOffset = true;
         Update();
      }
      UpdateCursor();
   }
}


/* Cursor nach rechts */
void EdLine::CursorRight()
{
   if(cursor < maxPos) {
      cursor++;
      if(cursor >= offset + width) {
         if(fastScroll == 0) {
            offset++;
         }
         else {
            offset += (width / fastScroll);
         }

         newOffset = true;
         Update();
      }
      UpdateCursor();
   }
}


/* Cursor auf Position 1 setzen */
void EdLine::CursorHome()
{
   if(cursor != 0) {
      cursor = 0;
      if(offset != 0) {
         offset = 0;
         newOffset = true;
      }
      Update();
   }
}


/* Cursor auf Zeilenende setzen */
void EdLine::CursorEnd()
{
   int c,o;

   c = strlen(text);
   if(cursor != c) {
      cursor = c;
 
      if(!( ((c >= offset) && (c < offset + width)) )) {
         if(c < width) {
            o = 0;
         }
         else {
            o = c - (width / 2);
         }

         if(offset != o) {
            offset    = o;
            newOffset = true;
         }
      }
      Update();
   }
}


// #######################################################################
// ===== Einfgen und Lschen von Zeichen ================================
// #######################################################################


/* Zeichen einfgen */
int EdLine::InsertChar(char chr)
{
   int i,l;
 
   l = strlen(text);      
   if((l < maxPos) && (cursor < maxPos)) {
      for(i = l;i < cursor;i++) {
         text[i] = ' ';
      }
      text[i]   = 0x00;
      Insert(chr,text,cursor);
    
      cursor++;
      if(cursor >= offset + width) {
         offset++;
         newOffset = true;
      }
      Update();
      return(true);
   }
   return(false);
}


/* Backspace */
void EdLine::Backspace()
{
   if(cursor > 0) {
      cursor--;
      if(cursor < offset) {
         offset--;
         newOffset = true;
      }
      Delete(text,cursor);
      Update();
   }
}


/* Zeichen lschen */
void EdLine::Del()
{
   Delete(text,cursor);
   Update();
}


// #######################################################################
// ===== Update ==========================================================
// #######################################################################


// Cursor neu zeichnen                 
void EdLine::UpdateCursor()
{
   if(sysCursorUpdate) {
      if(showCursor) {
         terminal->ShowSysCursor(leftEdge + cursor - offset,topEdge);
      }
      else {
         terminal->HideSysCursor();
      }
   }
}


/* Zeile darstellen */
void EdLine::Update()
{
   terminal->BeginWriting();
   terminal->WriteLine(leftEdge,topEdge,width,offset,blockStart,blockEnd,text);
   terminal->EndWriting();
   UpdateCursor();
}
