// ##########################################################################
// ####                                                                  ####
// ####      WarpEd - A portable text editor for OS/2 Warp and Linux     ####
// ####    ===========================================================   ####
// ####                                                                  ####
// #### row.h                                                            ####
// ####                                                                  ####
// #### Version 0.80  --  October 06, 2000                               ####
// ####                                                                  ####
// #### Copyright (C) 1996  Thomas Dreibholz                             ####
// ####               1997  Molbachweg 7                                 ####
// ####                     51674 Wiehl/Germany                          ####
// ####                     EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef ROW_H
#define ROW_H

#include <bool.h>

#include "terminal.h"


// Editierzeilen-Klasse
class EdLine
{
   // ----- Zeilen-Daten ------------------------
   public:
   int   leftEdge,topEdge,width;
   int   maxPos;
   int   offset;
   int   cursor;
   bool  newOffset;

   // ----- Zeilen-Einstellungen ----------------
   bool  showCursor;
   bool  fastScroll;
   bool  sysCursorUpdate;
   bool  isGood;

   // ----- Markierung --------------------------
   int   blockStart;
   int   blockEnd;

   // ----- Unterklassen ------------------------
   TerminalClass* terminal;


   // ===== Initialisierung ================================================
   public:
   EdLine();
   void init(TerminalClass* term,
          int x, int y, int w,
          int bufsize, int o, int c);
   ~EdLine();
   inline bool Good() const;
   inline bool Bad()  const;
   
   void        Resize(int x, int y, int w);     
   
    // ===== Zeilendaten ndern =============================================
   inline void SetText(char* src);
   inline void GetText(char* dest) const;
   inline void SetCursor(int pos);
   inline void SetOffset(int pos);
   inline void SetBlock(int start, int end);
     
   // ===== Zeilendaten abfragen ===========================================
   inline int  Cursor() const;
   inline int  Offset() const;
   inline void ShowCursor();
   inline void HideCursor();
   inline bool Changed();
   inline void SysCursorUpdate(int on);
      
   // ===== Cursor-Bewegungen ==============================================
   void   CursorLeft();
   void   CursorRight();
   void   CursorHome();
   void   CursorEnd();
  
   // ===== Zeichen einfgen und lschen ===================================
   int    InsertChar(char chr);
   void   Backspace();
   void   Del();
  
   // ===== Update des Displays ============================================
   void   UpdateCursor();
   void   Update();

   // ===== Einstellungen ndern ===========================================
   inline void SetFastScroll(int divisor);
   
   private:
   char* text;
};
 
#endif
